package com.intellij.jpdm.graph;

import com.intellij.jpdm.graph.nodes.*;
import com.intellij.jpdm.model.JpdlDomModelManager;
import com.intellij.jpdm.model.JpdlModel;
import com.intellij.jpdm.model.xml.*;
import com.intellij.jpdm.model.xml.wireObjectGroup.JavaActivity;
import com.intellij.openapi.application.Result;
import com.intellij.openapi.command.WriteCommandAction;
import com.intellij.openapi.graph.builder.GraphDataModel;
import com.intellij.openapi.graph.builder.NodesGroup;
import com.intellij.openapi.project.Project;
import com.intellij.openapi.util.text.StringUtil;
import com.intellij.psi.PsiFile;
import com.intellij.psi.xml.XmlFile;
import com.intellij.util.Function;
import com.intellij.util.containers.HashSet;
import com.intellij.util.xml.DomElement;
import com.intellij.util.xml.DomUtil;
import org.jetbrains.annotations.NonNls;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;

import java.util.*;

public class JpdlDataModel extends GraphDataModel<JpdlNode, JpdlEdge> {
    private final Collection<JpdlNode> myNodes = new HashSet<JpdlNode>();
    private final Collection<JpdlEdge> myEdges = new HashSet<JpdlEdge>();

    protected final Map<PsiFile, NodesGroup> myGroups = new HashMap<PsiFile, NodesGroup>();

    private final Project myProject;
    private final XmlFile myFile;
    @NonNls
    private static final String UNDEFINED_NAME = "Undefined";

    public JpdlDataModel(final XmlFile file) {
        myFile = file;
        myProject = file.getProject();
    }

    public Project getProject() {
        return myProject;
    }

    @NotNull
    public Collection<JpdlNode> getNodes() {
        return getNodes(true);
    }

    @NotNull
    public Collection<JpdlNode> getNodes(boolean refresh) {
        if (refresh) refreshDataModel();

        return myNodes;
    }

    @NotNull
    public Collection<JpdlEdge> getEdges() {
        return myEdges;
    }

    @NotNull
    public JpdlNode getSourceNode(final JpdlEdge jpdlBasicEdge) {
        return jpdlBasicEdge.getSource();
    }

    @NotNull
    public JpdlNode getTargetNode(final JpdlEdge jpdlBasicEdge) {
        return jpdlBasicEdge.getTarget();
    }

    @NotNull
    public String getNodeName(final JpdlNode jpdlBasicNode) {
        return "";//jpdlBasicNode.getName();
    }

    @NotNull
    public String getEdgeName(final JpdlEdge jpdlBasicEdge) {
        return jpdlBasicEdge.getName();
    }

    public JpdlEdge createEdge(@NotNull final JpdlNode from, @NotNull final JpdlNode to) {
        final String toName = to.getName();
        final DomElement element = from.getIdentifyingElement();
        if (element instanceof TransitionOwner) {
            final WriteCommandAction<JpdlEdge> action = new WriteCommandAction<JpdlEdge>(myProject) {
                protected void run(final Result<JpdlEdge> result) throws Throwable {
                    final Transition transition = ((TransitionOwner) element).addTransition();
                    transition.getTo().setStringValue(toName);
                    result.setResult(new JpdlBasicEdge(from, to, (Transition) transition.createStableCopy(), false));
                }
            };
            return action.execute().getResultObject();
        }
        return null;
    }

    public void dispose() {
    }


    private void refreshDataModel() {
        clearAll();

        updateDataModel();
    }

    private void clearAll() {
        myNodes.clear();
        myEdges.clear();
    }

    public void updateDataModel() {
        final ProcessDefinition processDefinition = getProcessDefinition();

        if (processDefinition == null) return;

        Map<String, List<JpdlNode>> allNodes = new HashMap<String, List<JpdlNode>>();

        addActivityNodes(processDefinition.getStarts(), new Function<Start, StartNode>() {
            public StartNode fun(Start start) {
                return new StartNode(start);
            }
        }, allNodes);

        addActivityNodes(processDefinition.getStates(), new Function<State, StateNode>() {
            public StateNode fun(State state) {
                return new StateNode(state);
            }
        }, allNodes);

        addActivityNodes(processDefinition.getDecisions(), new Function<Decision, DecisionNode>() {
            public DecisionNode fun(Decision decision) {
                return new DecisionNode(decision);
            }
        }, allNodes);

        addActivityNodes(processDefinition.getTasks(), new Function<Task, TaskNode>() {
            public TaskNode fun(Task task) {
                return new TaskNode(task);
            }
        }, allNodes);

        addActivityNodes(processDefinition.getForks(), new Function<Fork, ForkNode>() {
            public ForkNode fun(Fork fork) {
                return new ForkNode(fork);
            }
        }, allNodes);

        addActivityNodes(processDefinition.getJoins(), new Function<Join, JoinNode>() {
            public JoinNode fun(Join join) {
                return new JoinNode(join);
            }
        }, allNodes);

        addActivityNodes(processDefinition.getEnds(), new Function<End, EndActivityNode.EndNode>() {
            public EndActivityNode.EndNode fun(End end) {
                return new EndActivityNode.EndNode(end);
            }
        }, allNodes);

        addActivityNodes(processDefinition.getEndCancels(), new Function<EndCancel, EndActivityNode.EndCancelNode>() {
            public EndActivityNode.EndCancelNode fun(EndCancel end) {
                return new EndActivityNode.EndCancelNode(end);
            }
        }, allNodes);

        addActivityNodes(processDefinition.getEndErrors(), new Function<EndError, EndActivityNode.EndErrorNode>() {
            public EndActivityNode.EndErrorNode fun(EndError endError) {
                return new EndActivityNode.EndErrorNode(endError);
            }
        }, allNodes);

        addActivityNodes(processDefinition.getSubProcesses(), new Function<SubProcess, SubProcessNode>() {
            public SubProcessNode fun(SubProcess subProcess) {
                return new SubProcessNode(subProcess);
            }
        }, allNodes);

        addActivityNodes(processDefinition.getJavas(), new Function<JavaActivity, JavaNode>() {
            public JavaNode fun(JavaActivity javaActivity) {
                return new JavaNode(javaActivity);
            }
        }, allNodes);

        addActivityNodes(processDefinition.getCustoms(), new Function<Custom, CustomNode>() {
            public CustomNode fun(Custom custom) {
                return new CustomNode(custom);
            }
        }, allNodes);

        addActivityNodes(processDefinition.getSqls(), new Function<Sql, SqlNode>() {
            public SqlNode fun(Sql sql) {
                return new SqlNode(sql);
            }
        }, allNodes);

        addActivityNodes(processDefinition.getHqls(), new Function<Hql, HqlNode>() {
            public HqlNode fun(Hql hql) {
                return new HqlNode(hql);
            }
        }, allNodes);

        addActivityNodes(processDefinition.getRules(), new Function<Rules, RulesNode>() {
            public RulesNode fun(Rules rules) {
                return new RulesNode(rules);
            }
        }, allNodes);

        addActivityNodes(processDefinition.getRulesDecisions(), new Function<RulesDecision, RulesDecisionNode>() {
            public RulesDecisionNode fun(RulesDecision rulesDecision) {
                return new RulesDecisionNode(rulesDecision);
            }
        }, allNodes);

        addActivityNodes(processDefinition.getScripts(), new Function<Script, ScriptNode>() {
            public ScriptNode fun(Script script) {
                return new ScriptNode(script);
            }
        }, allNodes);

        addActivityNodes(processDefinition.getMails(), new Function<Mail, MailNode>() {
            public MailNode fun(Mail mail) {
                return new MailNode(mail);
            }
        }, allNodes);

        addActivityNodes(processDefinition.getGroups(), new Function<Group, GroupNode>() {
            public GroupNode fun(Group group) {
                return new GroupNode(group);
            }
        }, allNodes);


        for (List<? extends JpdlNode> sourceNodes : allNodes.values()) {
            for (JpdlNode sourceNode : sourceNodes) {
                final DomElement identifyingElement = sourceNode.getIdentifyingElement();
                if (identifyingElement instanceof TransitionOwner) {
                    final List<Transition> transitions = ((TransitionOwner) identifyingElement).getTransitions();

                    addTransitions(sourceNode, transitions, allNodes);
                }
            }
        }
    }

    private void addTransitions(final JpdlNode sourceNode,
                                final List<Transition> transitions,
                                final Map<String, List<JpdlNode>> allNodes) {
        for (Transition transition : transitions) {
            final String targetNodeName = transition.getTo().getStringValue();
            if (StringUtil.isEmptyOrSpaces(targetNodeName)) continue;

            final List<? extends JpdlNode> targetNodes = allNodes.get(targetNodeName);
            if (targetNodes != null) {
                for (JpdlNode targetNode : targetNodes) {
                    addTransition(new JpdlBasicEdge(sourceNode, targetNode, (Transition) transition.createStableCopy(), (targetNodes.size() > 1)));
                }
            }
        }
    }

    private void addNode(final JpdlNode node) {
        myNodes.add(node);
    }

    private void addTransition(final JpdlEdge edge) {
        myEdges.add(edge);
    }

    private <jpdlNode extends JpdlNode, jpdlActivity extends JpdlNamedActivity> void addActivityNodes(final List<jpdlActivity> activities, Function<jpdlActivity, jpdlNode> function, Map<String, List<JpdlNode>> nodes) {
        for (jpdlActivity activity : activities) {
            if (!DomUtil.hasXml(activity)) continue;

            String name = activity.getName().getStringValue();

            if (StringUtil.isEmptyOrSpaces(name)) name = UNDEFINED_NAME;
            if (!nodes.containsKey(name)) {
                nodes.put(name, new ArrayList<JpdlNode>());
            }
            JpdlNode node = function.fun(activity.<jpdlActivity>createStableCopy());

            nodes.get(name).add(node);

            addNode(node);
        }
    }

    @Nullable
    public ProcessDefinition getProcessDefinition() {
        final JpdlModel model = getModel();
        if (model == null || model.getRoots().size() != 1) return null;

        return model.getRoots().get(0).getRootElement();
    }

    public JpdlModel getModel() {
        return JpdlDomModelManager.getInstance(myProject).getJpdlModel(myFile);
    }
}
