package com.intellij.jpdm.graph.dnd;

import com.intellij.jpdm.JpdlIcons;
import com.intellij.jpdm.graph.JpdlDataModel;
import com.intellij.jpdm.graph.JpdlNode;
import com.intellij.jpdm.graph.JpdlNodeType;
import com.intellij.jpdm.graph.nodes.*;
import com.intellij.jpdm.model.xml.*;
import com.intellij.jpdm.model.xml.wireObjectGroup.JavaActivity;
import com.intellij.openapi.application.Result;
import com.intellij.openapi.command.WriteCommandAction;
import com.intellij.openapi.graph.builder.dnd.GraphDnDSupport;
import com.intellij.openapi.project.Project;
import com.intellij.openapi.util.Pair;
import com.intellij.util.Function;
import com.intellij.util.Icons;
import com.intellij.util.xml.DomUtil;

import javax.swing.*;
import java.util.LinkedHashMap;
import java.util.Map;

/**
 * User: Sergey.Vasiliev
 */
public class JpdlDnDSupport implements GraphDnDSupport<JpdlNode, JpdlNodeType> {
    private final JpdlDataModel myDataModel;
    private static final String UNKNOWN = "unknown";

    public JpdlDnDSupport(final JpdlDataModel dataModel) {
        myDataModel = dataModel;
    }


    public Map<JpdlNodeType, Pair<String, Icon>> getDnDActions() {
        Map<JpdlNodeType, Pair<String, Icon>> nodes = new LinkedHashMap<JpdlNodeType, Pair<String, Icon>>();

        nodes.put(JpdlNodeType.START, new Pair<String, Icon>("Start", JpdlIcons.START));
        nodes.put(JpdlNodeType.STATE, new Pair<String, Icon>("State", JpdlIcons.STATE));
        nodes.put(JpdlNodeType.TASK, new Pair<String, Icon>("Task", JpdlIcons.TASK));
        nodes.put(JpdlNodeType.JAVA, new Pair<String, Icon>("Java", Icons.CLASS_ICON));
        nodes.put(JpdlNodeType.DECISIION, new Pair<String, Icon>("Decision", JpdlIcons.DECISION));
        nodes.put(JpdlNodeType.FORK, new Pair<String, Icon>("Fork", JpdlIcons.FORK));
        nodes.put(JpdlNodeType.JOIN, new Pair<String, Icon>("Join", JpdlIcons.JOIN));
        nodes.put(JpdlNodeType.SUBPROCESS, new Pair<String, Icon>("Sub Process", JpdlIcons.SUB_PROCESS));
        nodes.put(JpdlNodeType.CUSTOM, new Pair<String, Icon>("Custom", JpdlIcons.CUSTOM));
        nodes.put(JpdlNodeType.SCRIPT, new Pair<String, Icon>("Script", JpdlIcons.SCRIPT));
        nodes.put(JpdlNodeType.SQL, new Pair<String, Icon>("Sql", JpdlIcons.SQL));
        nodes.put(JpdlNodeType.HQL, new Pair<String, Icon>("Hql", JpdlIcons.HQL));
        nodes.put(JpdlNodeType.RULES, new Pair<String, Icon>("Rules", JpdlIcons.RULES));
        nodes.put(JpdlNodeType.RULES_DECISION, new Pair<String, Icon>("Rules Decision", JpdlIcons.RULES_DECISION));
        nodes.put(JpdlNodeType.MAIL, new Pair<String, Icon>("Mail", JpdlIcons.MAIL));
        nodes.put(JpdlNodeType.GROUP, new Pair<String, Icon>("Group", JpdlIcons.GROUP));

        nodes.put(JpdlNodeType.END, new Pair<String, Icon>("End", JpdlIcons.END));
        nodes.put(JpdlNodeType.END_CANCEL, new Pair<String, Icon>("End Cancel", JpdlIcons.END_CANCEL));
        nodes.put(JpdlNodeType.END_ERROR, new Pair<String, Icon>("End Error", JpdlIcons.END_ERROR));


        return nodes;
    }

    public boolean canStartDragging(final JpdlNodeType jpdlNodeType) {
        final ProcessDefinition processDefinition = getDataModel().getProcessDefinition();
        if (processDefinition != null) {
            if (jpdlNodeType == JpdlNodeType.START) {
                return processDefinition.getStarts().size() == 0;
            }
        }
        return true;
    }

    public JpdlNode drop(final JpdlNodeType jpdlNodeType) {
        final ProcessDefinition processDefinition = getDataModel().getProcessDefinition();

        if (processDefinition == null) return null;

        final Project project = getDataModel().getProject();
        switch (jpdlNodeType) {
            case STATE:
                return startInWCA(project, processDefinition, getDropStateFunction());
            case TASK:
                return startInWCA(project, processDefinition, getDropTaskFunction());
            case FORK:
                return startInWCA(project, processDefinition, getDropForkFunction());
            case JOIN:
                return startInWCA(project, processDefinition, getDropJoinFunction());
            case JAVA:
                return startInWCA(project, processDefinition, getDropJavaFunction());
            case CUSTOM:
                return startInWCA(project, processDefinition, getDropCustomFunction());
            case DECISIION:
                return startInWCA(project, processDefinition, getDropDecisionFunction());
            case START:
                return startInWCA(project, processDefinition, getDropStartFunction());
            case SUBPROCESS:
                return startInWCA(project, processDefinition, getDropSubProcessFunction());
            case END:
                return startInWCA(project, processDefinition, getDropEndFunction());
            case MAIL:
                return startInWCA(project, processDefinition, getDropMailFunction());
            case END_CANCEL:
                return startInWCA(project, processDefinition, getDropEndCancelFunction());
            case END_ERROR:
                return startInWCA(project, processDefinition, getDropEndErrorFunction());
            case SCRIPT:
                return startInWCA(project, processDefinition, getDropScriptFunction());
            case RULES:
                return startInWCA(project, processDefinition, getDropRuleFunction());
            case RULES_DECISION:
                return startInWCA(project, processDefinition, getDropRulesDecisionFunction());
            case SQL:
                return startInWCA(project, processDefinition, getDropSqlFunction());
            case HQL:
                return startInWCA(project, processDefinition, getDropHqlFunction());
            case GROUP:
                return startInWCA(project, processDefinition, getDropGroupFunction());
            default:
                break;
        }


        return null;
    }

    private Function<ProcessDefinition, JpdlNode> getDropForkFunction() {
        return new Function<ProcessDefinition, JpdlNode>() {
            public JpdlNode fun(final ProcessDefinition processDefinition) {
                final Fork fork = processDefinition.addFork();
                fork.getName().setStringValue(UNKNOWN);
                return new ForkNode((Fork) fork.createStableCopy());
            }
        };
    }

    private Function<ProcessDefinition, JpdlNode> getDropJoinFunction() {
        return new Function<ProcessDefinition, JpdlNode>() {
            public JpdlNode fun(final ProcessDefinition processDefinition) {
                final Join join = processDefinition.addJoin();
                join.getName().setStringValue(UNKNOWN);
                return new JoinNode((Join) join.createStableCopy());
            }
        };
    }

    private Function<ProcessDefinition, JpdlNode> getDropJavaFunction() {
        return new Function<ProcessDefinition, JpdlNode>() {
            public JpdlNode fun(final ProcessDefinition processDefinition) {
                final JavaActivity javaActivity = processDefinition.addJava();
                javaActivity.getName().setStringValue(UNKNOWN);
                return new JavaNode((JavaActivity) javaActivity.createStableCopy());
            }
        };
    }

    private Function<ProcessDefinition, JpdlNode> getDropCustomFunction() {
        return new Function<ProcessDefinition, JpdlNode>() {
            public JpdlNode fun(final ProcessDefinition processDefinition) {
                final Custom custom = processDefinition.addCustom();
                custom.getName().setStringValue(UNKNOWN);
                return new CustomNode((Custom) custom.createStableCopy());
            }
        };
    }

    private Function<ProcessDefinition, JpdlNode> getDropScriptFunction() {
        return new Function<ProcessDefinition, JpdlNode>() {
            public JpdlNode fun(final ProcessDefinition processDefinition) {
                final Script script = processDefinition.addScript();
                script.getName().setStringValue(UNKNOWN);
                return new ScriptNode((Script) script.createStableCopy());
            }
        };
    }

    private Function<ProcessDefinition, JpdlNode> getDropSqlFunction() {
        return new Function<ProcessDefinition, JpdlNode>() {
            public JpdlNode fun(final ProcessDefinition processDefinition) {
                final Sql sql = processDefinition.addSql();
                sql.getName().setStringValue(UNKNOWN);
                return new SqlNode((Sql) sql.createStableCopy());
            }
        };
    }

    private Function<ProcessDefinition, JpdlNode> getDropHqlFunction() {
        return new Function<ProcessDefinition, JpdlNode>() {
            public JpdlNode fun(final ProcessDefinition processDefinition) {
                final Hql hql = processDefinition.addHql();
                hql.getName().setStringValue(UNKNOWN);
                return new HqlNode((Hql) hql.createStableCopy());
            }
        };
    }
    private Function<ProcessDefinition, JpdlNode> getDropGroupFunction() {
        return new Function<ProcessDefinition, JpdlNode>() {
            public JpdlNode fun(final ProcessDefinition processDefinition) {
                final Group group = processDefinition.addGroup();
                group.getName().setStringValue(UNKNOWN);
                return new GroupNode((Group) group.createStableCopy());
            }
        };
    }

    private Function<ProcessDefinition, JpdlNode> getDropRuleFunction() {
        return new Function<ProcessDefinition, JpdlNode>() {
            public JpdlNode fun(final ProcessDefinition processDefinition) {
                final Rules rules = processDefinition.addRules();
                rules.getName().setStringValue(UNKNOWN);
                return new RulesNode((Rules) rules.createStableCopy());
            }
        };
    }
    private Function<ProcessDefinition, JpdlNode> getDropRulesDecisionFunction() {
        return new Function<ProcessDefinition, JpdlNode>() {
            public JpdlNode fun(final ProcessDefinition processDefinition) {
                final RulesDecision rulesDecision = processDefinition.addRulesDecision();
                rulesDecision.getName().setStringValue(UNKNOWN);
                return new RulesDecisionNode((RulesDecision) rulesDecision.createStableCopy());
            }
        };
    }

    private static Function<ProcessDefinition, JpdlNode> getDropEndFunction() {
        return new Function<ProcessDefinition, JpdlNode>() {
            public JpdlNode fun(final ProcessDefinition processDefinition) {
                final End end = processDefinition.addEnd();
                end.getName().setStringValue(UNKNOWN);
                return new EndActivityNode.EndNode((End) end.createStableCopy());
            }
        };
    }

    private static Function<ProcessDefinition, JpdlNode> getDropEndCancelFunction() {
        return new Function<ProcessDefinition, JpdlNode>() {
            public JpdlNode fun(final ProcessDefinition processDefinition) {
                final EndCancel endCancel = processDefinition.addEndCancel();
                endCancel.getName().setStringValue(UNKNOWN);
                return new EndActivityNode.EndCancelNode((EndCancel) endCancel.createStableCopy());
            }
        };
    }

    private static Function<ProcessDefinition, JpdlNode> getDropEndErrorFunction() {
        return new Function<ProcessDefinition, JpdlNode>() {
            public JpdlNode fun(final ProcessDefinition processDefinition) {
                final EndError endError = processDefinition.addEndError();
                endError.getName().setStringValue(UNKNOWN);
                return new EndActivityNode.EndErrorNode((EndError) endError.createStableCopy());
            }
        };
    }

    private static Function<ProcessDefinition, JpdlNode> getDropStartFunction() {
        return new Function<ProcessDefinition, JpdlNode>() {
            public JpdlNode fun(final ProcessDefinition processDefinition) {
                final Start start = processDefinition.addStart();
                start.getName().setStringValue(UNKNOWN);
                return new StartNode((Start) start.createStableCopy());
            }
        };
    }

    private static Function<ProcessDefinition, JpdlNode> getDropTaskFunction() {
        return new Function<ProcessDefinition, JpdlNode>() {
            public JpdlNode fun(final ProcessDefinition processDefinition) {
                final Task task = processDefinition.addTask();
                task.getName().setStringValue(UNKNOWN);
                return new TaskNode((Task) task.createStableCopy());
            }
        };
    }

    private static Function<ProcessDefinition, JpdlNode> getDropSubProcessFunction() {
        return new Function<ProcessDefinition, JpdlNode>() {
            public JpdlNode fun(final ProcessDefinition processDefinition) {
                final SubProcess processState = processDefinition.addSubProcess();
                processState.getName().setStringValue(UNKNOWN);
                return new SubProcessNode((SubProcess) processState.createStableCopy());
            }
        };
    }

    private static Function<ProcessDefinition, JpdlNode> getDropStateFunction() {
        return new Function<ProcessDefinition, JpdlNode>() {
            public JpdlNode fun(final ProcessDefinition processDefinition) {
                final State state = processDefinition.addState();

                state.getName().setStringValue(UNKNOWN);

                return new StateNode((State) state.createStableCopy());
            }
        };
    }
    private static Function<ProcessDefinition, JpdlNode> getDropMailFunction() {
        return new Function<ProcessDefinition, JpdlNode>() {
            public JpdlNode fun(final ProcessDefinition processDefinition) {
                final Mail mail = processDefinition.addMail();

                mail.getName().setStringValue(UNKNOWN);

                return new MailNode((Mail) mail.createStableCopy());
            }
        };
    }

    private static Function<ProcessDefinition, JpdlNode> getDropDecisionFunction() {
        return new Function<ProcessDefinition, JpdlNode>() {
            public JpdlNode fun(final ProcessDefinition processDefinition) {
                final Decision decision = processDefinition.addDecision();
                decision.getName().setStringValue(UNKNOWN);
                return new DecisionNode((Decision) decision.createStableCopy());
            }
        };
    }

    private static JpdlNode startInWCA(final Project project,
                                       final ProcessDefinition processDefinition,
                                       final Function<ProcessDefinition, JpdlNode> function) {
        return new WriteCommandAction<JpdlNode>(project, DomUtil.getFile(processDefinition)) {
            protected void run(final Result<JpdlNode> result) throws Throwable {
                result.setResult(function.fun(processDefinition));
            }
        }.execute().getResultObject();

    }

    public JpdlDataModel getDataModel() {
        return myDataModel;
    }
}
